#!/usr/bin/env python3

from requests import get

from . import api
from .._common.endpoint import BaseURL, Endpoint
from .._common.response import APIResponse


class SessionserverEndpoint(Endpoint):
    BASE_URL = BaseURL('https://sessionserver.mojang.com')
    UUID_TO_PROFILE = '/session/minecraft/profile/{uuid}'
    BLOCKED_SERVERS = '/blockedservers'


def get_user_profile(player):
    response = get(
        SessionserverEndpoint.UUID_TO_PROFILE.url.format(uuid=player.uuid))
    return APIResponse(response)


def get_blocked_servers():
    response = get(SessionserverEndpoint.BLOCKED_SERVERS.url)
    return APIResponse(response)



#!/usr/bin/env python3

from functools import wraps

from ..user.player import Player
from ..utils.uuid import is_valid_uuid


def accept_player():
    def decorator(func):
        @wraps(func)
        def with_player_acceptance(*args, **kwargs):
            args = list(args)
            for arg_pos in range(len(args)):
                player = args[arg_pos]
                if isinstance(player, Player):
                    pass
                elif isinstance(player, str):
                    if is_valid_uuid(player):
                        args[arg_pos]=Player(uuid=player)
                    else:
                        args[arg_pos]=Player(username=player)
                else:
                    raise TypeError(
                        'player must be a valid Player, username, or UUID')

            return func(*args, **kwargs)

        return with_player_acceptance

    return decorator



#!/usr/bin/env python3

class APIResponse:
    def __new__(cls, response, *args, **kwargs):
        data = None
        try:
            data = response.json()
        except ValueError:
            return {'response': response}
        else:
            if isinstance(data, dict):
                return {'response': response, 'data': dict(data)}
            elif isinstance(data, list):
                return {'response': response, 'data': list(data)}
            else:
                raise TypeError(
                    'response\'s JSON data must be of type \'dict\' or \'list\'')



#!/usr/bin/env python3

from requests import delete, get, post, put

from .._common.endpoint import BaseURL, Endpoint
from .._common.response import APIResponse


class APIEndpoint(Endpoint):
    BASE_URL = BaseURL('https://api.mojang.com')
    USERNAME_TO_UUID_AT_TIME = '/users/profiles/minecraft/{username}'
    UUID_TO_USERNAME_HISTORY = '/user/profiles/{uuid}/names'
    USERNAMES_TO_UUIDS = '/profiles/minecraft'
    CHANGE_SKIN = '/user/profile/{uuid}/skin'
    UPLOAD_SKIN = '/user/profile/{uuid}/skin'
    RESET_SKIN = '/user/profile/{uuid}/skin'
    STATISTICS = '/orders/statistics'

def get_uuid(player, timestamp=None):
    params = {
        'at': timestamp
    }
    response = get(APIEndpoint.USERNAME_TO_UUID_AT_TIME.url.format(
        username=player.username), params=params)
    return APIResponse(response)

def get_username_history(player):
    response = get(
        APIEndpoint.UUID_TO_USERNAME_HISTORY.url.format(uuid=player.uuid))
    return APIResponse(response)

def change_skin(player, access_token, skin_url, slim_model=False):
    headers = {
        'Authorization': 'Bearer ' + access_token
    }
    payload = {
        'model': 'slim' if slim_model else '',
        'url': skin_url
    }
    response = post(APIEndpoint.CHANGE_SKIN.url.format(
        uuid=player.uuid), headers=headers, data=payload)
    return APIResponse(response)

def upload_skin(player, access_token, path_to_skin, slim_model=False):
    headers = {
        'Authorization': 'Bearer ' + access_token
    }
    files = {
        'model': 'slim' if slim_model else '',
        'file': open(path_to_skin, 'rb')
    }
    response = put(APIEndpoint.UPLOAD_SKIN.url.format(
        uuid=player.uuid), headers=headers, files=files)
    return APIResponse(response)

def reset_skin(player, access_token):
    headers = {
        'Authorization': 'Bearer ' + access_token
    }
    response = delete(APIEndpoint.RESET_SKIN.url.format(
        uuid=player.uuid), headers=headers)
    return APIResponse(response)


def get_statistics(item_sold_minecraft=False, prepaid_card_redeemed_minecraft=False, item_sold_cobalt=False, item_sold_scrolls=False):
    sales_mapping = {
        'item_sold_minecraft': item_sold_minecraft,
        'prepaid_card_redeemed_minecraft': prepaid_card_redeemed_minecraft,
        'item_sold_cobalt': item_sold_cobalt,
        'item_sold_scrolls': item_sold_scrolls
    }
    payload = {
        'metricKeys': [k for (k, v) in sales_mapping.items() if v]
    }
    response = post(APIEndpoint.STATISTICS.url, json=payload)
    return APIResponse(response)



#!/usr/bin/env python3

from requests import get

from .._common.endpoint import BaseURL, Endpoint
from .._common.response import APIResponse


class StatusEndpoint(Endpoint):
    BASE_URL = BaseURL('https://status.mojang.com')
    CHECK = '/check'


def get_status():
    response = get(StatusEndpoint.CHECK.url)
    return APIResponse(response)



#!/usr/bin/env python3

from .servers.api import get_statistics
from .servers.sessionserver import get_blocked_servers
from .servers.status import get_status
from .user.player import Player
from .utils.uuid import generate_client_token



#!/usr/bin/env python3
from ..servers.api import (get_uuid, get_username_history, change_skin, upload_skin, reset_skin)
from ..servers.authserver import (authenticate_user, invalidate_access_token,
                                 refresh_access_token, signout_user,
                                 validate_access_token)
from ..servers.sessionserver import get_user_profile

class Player:
    def __init__(self, username='', uuid=''):
        self._validate(username, uuid)
        self._username = username
        self._uuid = uuid
        self._access_token=None
        self._client_token=None
        self._get_player()

    def _validate(self, username=None, uuid=None):
        if username is None:
            username = self.username

        if uuid is None:
            uuid = self.uuid

        if not (username or uuid):
            raise AttributeError('Player must contain a username or UUID')

    def _get_player(self):
        if not self.uuid and self.username:
            self.uuid = get_uuid(self)['data']['id']
        if not self.username and self.uuid:
            self.username = get_user_profile(self)['data']['name']

    def authenticate(self, username, password):
        resp=authenticate_user(username, password)
        if resp['response'].status_code == 200:
            data=resp['data']
            current_profile=data['selectedProfile']
            if current_profile:
                self._username=current_profile['name']
                self._uuid=current_profile['id']
                self._access_token=data['accessToken']
                self._client_token=data['clientToken']
                return True, resp
        return False, resp

    def signout(self):
        if self.is_authenticated:
            return invalidate_access_token(self._access_token, self._client_token)['response'].status_code == 204
        return False

    @property
    def is_authenticated(self):
        if self._access_token != None and self._client_token != None != None:
            return self.valid_tokens()
        return False

    @property
    def username(self):
        return self._username

    @username.setter
    def username(self, username):
        if username:
            self._username = username
        else:
            del self.username

    @username.deleter
    def username(self):
        self._validate(username='')
        self._username = ''

    @property
    def uuid(self):
        return self._uuid

    @uuid.setter
    def uuid(self, uuid):
        if uuid:
            self._uuid = uuid
        else:
            del self.uuid

    @uuid.deleter
    def uuid(self):
        self._validate(uuid='')
        self._uuid = ''

    @property
    def access_token(self):
        return self._access_token

    @property
    def tokens(self):
        return (self._access_token, self.clientToken)

    @tokens.setter
    def tokens(self, tokens):
        if tokens and len(tokens)==2 and tokens[0] and tokens[1]:
            if self._valid_tokens(tokens[0], tokens[1]):
                return self._refresh_tokens(tokens[0], tokens[1])
        else:
            del self.access_token
            del self.client_token
        return None

    def valid_tokens(self):
        return self._valid_tokens(self._access_token, self._client_token)

    def _valid_tokens(self, accessToken, clientToken):
        return validate_access_token(accessToken, clientToken)['response'].status_code==204

    def refresh_tokens(self):
        return self._refresh_tokens(self._access_token, self._client_token)

    def _refresh_tokens(self, access_token, client_token):
        resp = refresh_access_token(access_token, client_token, request_user=True)
        if resp['response'].status_code == 200:
            data=resp['data']
            self._access_token = data['accessToken']
            self._client_token = data['clientToken']
            self._username = data['selectedProfile']['name']
            self._uuid = data['selectedProfile']['id']
            return resp
        return None

    @access_token.deleter
    def access_token(self):
        if self._access_token and self._client_token:
            invalidate_access_token(self._access_token, self._client_token)
        self._access_token=None
        self._client_token=None

    @property
    def client_token(self):
        return self._client_token

    @client_token.deleter
    def client_token(self):
        if self._access_token and self._client_token:
            invalidate_access_token(self._access_token, self._client_token)
        self._access_token=None
        self._client_token=None

    @property
    def profile(self):
        return get_user_profile(self)

    def username_history(self):
        return get_username_history(self)

    def change_skin(self, skin_url, slim_model=False):
        assert self.is_authenticated, "Player must be authenticated to perform this action"
        return change_skin(player=self, access_token=self._access_token, skin_url=skin_url, slim_model=slim_model)

    def upload_skin(self, path_to_skin, slim_model=False):
        assert self.is_authenticated, "Player must be authenticated to perform this action"
        return upload_skin(player=self, access_token=self._access_token, path_to_skin=path_to_skin, slim_model=slim_model)

    def reset_skin(self):
        assert self.is_authenticated, "Player must be authenticated to perform this action"
        return reset_skin(player=self, access_token=self._access_token)



#!/usr/bin/env python3

from requests import post

from .._common.endpoint import BaseURL, Endpoint
from .._common.response import APIResponse
from ..utils.uuid import generate_client_token


class AuthserverEndpoint(Endpoint):
    BASE_URL = BaseURL('https://authserver.mojang.com')
    AUTHENTICATE = '/authenticate'
    REFRESH = '/refresh'
    VALIDATE = '/validate'
    SIGNOUT = '/signout'
    INVALIDATE = '/invalidate'


def authenticate_user(username, password, client_token=generate_client_token(), request_user=False):
    payload = {
        'agent': {
            'name': 'Minecraft',
            'version': 1
        },
        'username': username,
        'password': password,
        'clientToken': client_token
    }
    if request_user:
        payload['requestUser'] = True

    response = post(AuthserverEndpoint.AUTHENTICATE.url, json=payload)
    return APIResponse(response)


def refresh_access_token(access_token, client_token, request_user=False):
    payload = {
        'accessToken': access_token,
        'clientToken': client_token
    }
    if request_user:
        payload['requestUser'] = True

    response = post(AuthserverEndpoint.REFRESH.url, json=payload)
    return APIResponse(response)


def validate_access_token(access_token, client_token=None):
    payload = {
        'accessToken': access_token
    }
    if client_token != None:
        payload['clientToken'] = client_token

    response = post(AuthserverEndpoint.VALIDATE.url, json=payload)
    return APIResponse(response)


def signout_user(username, password):
    payload = {
        'username': username,
        'password': password
    }
    response = post(AuthserverEndpoint.SIGNOUT.url, json=payload)
    return APIResponse(response)


def invalidate_access_token(access_token, client_token):
    payload = {
        'accessToken': access_token,
        'clientToken': client_token
    }
    response = post(AuthserverEndpoint.INVALIDATE.url, json=payload)
    return APIResponse(response)



#!/usr/bin/env python3

from codecs import open
from os import path

from setuptools import find_packages, setup

here = path.abspath(path.dirname(__file__))
with open(path.join(here, 'README.rst'), encoding='utf-8') as readme:
    long_description = readme.read()

setup(
    name='mojang-api-3',
    version='3.0.3',
    description='A full Python wrapper of Mojang\'s API and Authentication API.',
    long_description=long_description,
    url='https://github.com/Kiln707/mojang-api',
    author='Synchronous, Kiln707',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6'
    ],
    keywords='mojang minecraft mc api authentication auth',
    packages=find_packages(),
    setup_requires='pytest-runner',
    tests_require=[
        'pytest',
        'pytest-cov'
    ],
    install_requires=[
        'python-box',
        'requests'
    ],
    python_requires='>=3.5.*'
)



#!/usr/bin/env python3

from enum import Enum


class BaseURL:
    def __init__(self, base_url=''):
        self._base_url = base_url

    def __get__(self, instance, owner):
        return self._base_url


class Endpoint(Enum):
    BASE_URL = BaseURL()

    def __init__(self, endpoint_uri):
        self._endpoint_uri = endpoint_uri
        self._url = self.BASE_URL + self.endpoint_uri

    @property
    def endpoint_uri(self):
        return self._endpoint_uri

    @property
    def url(self):
        return self._url



#!/usr/bin/env python3

from uuid import UUID, uuid4


def generate_client_token():
    return uuid4().hex


def is_valid_uuid(uuid_string):
    try:
        UUID(uuid_string)
    except ValueError:
        return False

    return True
